set.seed(0)

source('https://waxworksmath.com/Authors/G_M/Luenberger/Investment_Science/Code/Chapter12/utils.R')
source('./utils.R')

mA = 0.0001
sA = 0.017

## Part (a):
##
S0 = 115
r = 0.02
T = 90/365 ## years
K = 113
sigma = sA * sqrt(252) ## convert per-day units to per-year units 

d1 = (log(S0/K) + (r + 0.5 * sigma^2) * T)/(sigma * sqrt(T))
d2 = d1 - sigma*sqrt(T)
C = pnorm(d1)*S0 - pnorm(d2)*K*exp(-r*T)
print(sprintf('Part (a): d1= %f; d2= %f; Call value= %f', d1, d2, C))

## Part (b):
##
delta = pnorm(d1)
print(sprintf('Part (b): delta= %f', delta))

## Part (c):
##
La = delta * ( S0 / C ) 
print(sprintf('Part (c): La= %f', La))

## Part (d):
##
S = 1000
alpha = 0.05

VaR_underlying = VaR_norm(alpha, mA, sA, S)
VaR_option = La * VaR_underlying
print(sprintf('Part (d): VaR_underlying= %f; VaR_option= %f', VaR_underlying, VaR_option))

## Part (e):
##
w = 1000/(1000+1000)
VaR_P_pt_e = ( w + (1-w) * La ) * 2 * VaR_underlying ## <- note the factor of 2 since the initial portfolio is 1000 + 1000 = 2000 = 2 x 1000 
print(sprintf('Part (e): VaR_porfolio= %f', VaR_P_pt_e))

## Part (f):
##
mB = 0.0002
sB = 0.019
rhoAB = 0.3

w = 1/2
mu_P_hat = w * mB + (1-w) * La * mA
sigma_P_hat = sqrt( w^2 * sB^2 + (1-w)^2 * La^2 * sA^2 + 2 * w * (1-w) * La * rhoAB * sA * sB )
VaR_P_pt_f = VaR_norm(alpha, mu_P_hat, sigma_P_hat, 2 * S) ## <- note the factor of 2 
print(sprintf('Part (f): mu_P_hat= %f; sigma_P_hat= %f; VaR_porfolio= %f', mu_P_hat, sigma_P_hat, VaR_P_pt_f))

## Part (g):
##
w = 1/3
mu_P_hat = w * La * mA + w * mA + w * mB
sigma_P_hat = sqrt( w^2 * ( La + 1 )^2 * sA^2 + w^2 * sB^2 + 2 * w^2 * ( La + 1 ) * rhoAB * sA * sB )
VaR_P_pt_g = VaR_norm(alpha, mu_P_hat, sigma_P_hat, 3 * S) ## <- note the factor of 3 as we have 3 assets at $1000 each 
print(sprintf('Part (g): mu_P_hat= %f; sigma_P_hat= %f; VaR_porfolio= %f', mu_P_hat, sigma_P_hat, VaR_P_pt_g))
