
save_plots = FALSE

set.seed(0) 

## Get price/return data from Ruppert's other book:
##
dat = read.csv("../../../SNDAFFE_2nd_Edition/BookCode/Data/Stock_Bond_2004_to_2006.csv", header=T)

## Get the information on Ford (F) and S&P 500 Index:
##
prices = dat[,c(7,24)]
n = dim(prices)[1] 

dat2 = as.matrix( cbind( dat[(2:n),3]/365, 100*(prices[2:n,]/prices[1:(n-1),] - 1) ) ) ## units are now percents
colnames(dat2)[1] = "treasury" 
risk_free = dat2[,1]
ExRet = dat2[,2:3] - risk_free ## excess return 
market = ExRet[,2] ## the markets excess return 
stockExRet = ExRet[,1] ## Fords excess return 

## Lets compute the beta the "normal" way:
##
## R_{j,t} = mu_{f,t} + beta_j ( R_{M,t} - mu_{f,t} ) + epsilon_{j,t}
##
fit_reg = lm( stockExRet ~ market ) 
print(summary(fit_reg))

## Using least squares lets compute the desired spline:
##
y = stockExRet
x = market

knots = quantile(x, probs=c(0.25, 0.5, 0.75))

## Build the components of the spline:
##
extraTerm1 = (x-knots[1])^2 * (x > knots[1])
extraTerm2 = (x-knots[2])^2 * (x > knots[2])
extraTerm3 = (x-knots[3])^2 * (x > knots[3])

m = lm(y ~ 1 + x + I(x^2) + extraTerm1 + extraTerm2 + extraTerm3)
print(summary(m))



