import numpy as np
from numpy import sqrt

from cvxopt import matrix, solvers

import pylab

def demo_cvxopt():
    #
    # Exercise cvxopt: https://medium.com/tech-that-works/solving-quadratic-convex-optimization-problems-in-python-e378c7442494
    #
    P = 2*matrix([ [3., 1.], [1., 4.] ])
    q = matrix([5., 1.])
    G = matrix([[-1.0,0.0],[0.0,-1.0]])
    h = matrix([0.0,0.0])
    A = matrix([1.0, 2.0], (1,2))
    b = matrix(1.0)
    sol=solvers.qp(P, q, G, h, A, b)
    print(sol['x'])


if __name__ == '__main__':
    #
    #
    if False:
        print('Running a demo of cvxopt:')
        demo_cvxopt()

    mu_f = 0.028
    mu = matrix([0.1, 0.004, 0.07, 0.11, 0.05])
    P = Omega = matrix([[0.38, 0.02, 0.01, 0.0, 0.0],
                        [0.02, 0.17, 0.03, 0.01, 0.02],
                        [0.01, 0.03, 0.38, 0.04, 0.05],
                        [0.0, 0.01, 0.04, 0.47, 0.03],
                        [0.0, 0.02, 0.05, 0.03, 0.26]])

    N = mu.size[0]
    bone = matrix(np.ones((N, 1))) # a column vector of 1's

    # Fill in the matrices we need to create for a call to the solver:
    #
    q = matrix(np.zeros((N, 1)))

    A = matrix(np.vstack((mu.T, bone.T)))

    if False:
        # Restrict the weights to be nonnegative only:
        G = matrix(-np.identity(N))
        h = matrix(np.zeros((N, 1)))
    else:
        # Restrict the weights such that: 0 <= w_i <= 0.33
        G = matrix(np.vstack((-np.identity(N), np.identity(N))))
        UL = 0.33
        h = matrix(np.vstack((np.zeros((N, 1)), UL*np.ones((N, 1)))))

    solvers.options['show_progress'] = False

    #target_mus = np.linspace(min(mu), max(mu), 100)
    target_mus = np.linspace(0.055, max(mu), 200) # a study of the minium variance portfolio gives 0.055 for the smallest mean 
    mu_P, sigma_P = [], [] # save the target mu's we could compute a valid solution for (and their standard deviations)
    ws = [] # save the valid weights 
    for m in target_mus:
        b = matrix([m, 1.0])
        sol = solvers.qp(P, q, G, h, A, b)
        if sol['status']!='optimal': continue # some setting for mu_P are not possible
        mu_P.append(m)
        s = sqrt( sol['x'].T * Omega * sol['x'] )[0, 0] 
        sigma_P.append(s)
        ws.append(np.asarray(sol['x']))
    mu_P = np.asarray(mu_P)
    sigma_P = np.asarray(sigma_P)

    # Find the minimum variance portfolio:
    #
    mv_index = np.argmin(sigma_P)
    mv_sigma = sigma_P[mv_index]
    mv_mu = mu_P[mv_index]
    print('min_variance_sigma= {0:8.3f}; min_variance_mean= {1:8.3f}'.format(mv_sigma, mv_mu))

    efficient_frontier = mu_P >= mv_mu
    sharpe_ratio = (mu_P - mu_f)/sigma_P
    tangent_index = np.argmax(sharpe_ratio)
    
    pylab.plot(sigma_P[efficient_frontier], mu_P[efficient_frontier], '-', color='black', label='efficient frontier')
    pylab.plot([0.0], [mu_f], 'x', color='black', label='risk free asset')
    pylab.plot(sigma_P[tangent_index], mu_P[tangent_index], '*', color='blue', label='tangency portfolio')
    pylab.plot(sigma_P[mv_index], mu_P[mv_index], 'o', color='green', label='minimum variance portfolio')
    pylab.plot([0.0, sigma_P[tangent_index]], [mu_f, mu_P[tangent_index]], '.-')
    pylab.grid()
    pylab.xlabel('standard deviation of return')
    pylab.ylabel('expected return')
    pylab.legend()
    pylab.show()

    
