#
# Written by:
# -- 
# John L. Weatherwax                2009-04-21
# 
# email: wax@alum.mit.edu
# 
# Please send comments and especially bug reports to the
# above email address.
# 
#-----

set.seed(0)

# The number of data points and the degree of freedoms:
total_df = 14
n = total_df + 1
regression_df = 2
error_df = total_df - regression_df

f_value = qf( 1 - 0.05, regression_df, error_df )

error_ss = 3.25
error_ms = error_ss / error_df
sigma2 = error_ms

# Since F = MS / sigma^2 we can solve for MS to get MS = F * sigma^2
# 
regression_ms = sigma2 * f_value 
regression_ss = regression_ms * regression_df
total_ss = regression_ss + error_ss 

R2 = regression_ss / total_ss 

print(sprintf('regression SS= %f; regression MS= %f; f_value= %f', regression_ss, regression_ms, f_value))
print(sprintf('error MS= %f; total SS= %f; R^2= %f', error_ms, total_ss, R2))


# Verify the above calculations on some other ANOVA tables:
#
set.seed(0)

# Two variable prediction:
# 
X1 = rnorm(15)
X2 = rnorm(15)
Y = 1*X1 - 2*X2 + rnorm(15)
m = lm( Y ~ X1 + X2 )
anova(m)

F_X1_value = qf( 1 - 0.0006981, 1, 12 ) # take first Pr(>F) value
F_X2_value = qf( 1 - 0.0005173, 1, 12 ) # take second Pr(>F) value 

# Simple linear regression:
# 
m = lm( Y ~ X1 )
anova(m) 
F_X1_value = qf( 1 - 0.01519, 1, 13 ) 

s2 = 2.0608 # extract out estimate of the error variance (Residuals Mean Sq element)
Mean_Sq_X1 = F_X1_value * s2


